@@ -1,6 +1,6 @@
 #-------------------------------------------------------------------------------
-# Lint, compile, install, start an Android App using just the command line tools
-# without using ant or gradle, just 100% Pure Perl
+# Compile, install, start an Android App using the sdk command line build tools
+# rather than calling them via ant or gradle.
 # Philip R Brenan at gmail dot com, Appa Apps Ltd, 2017
 #-------------------------------------------------------------------------------
 
@@ -14,7 +14,7 @@ use Data::Table::Text qw(:all);
 use File::Copy;
 use POSIX qw(strftime);                                                         # http://www.cplusplus.com/reference/ctime/strftime/
 
-our $VERSION = '2017.444';
+our $VERSION = '2017.4444';
 
 #-------------------------------------------------------------------------------
 # Constants
@@ -37,7 +37,7 @@ sub new()
         version    =>$version};
  }
 
-if (1)                                                                          # Parameters that can be set by the caller
+if (1)                                                                          # Parameters that can be set by the caller - see the pod at the end of this file for a complete description of what each parameter does
  {Data::Table::Text::genLValueScalarMethods(
   qw(action),                                                                   # Optional: Default action to perform, compile, lint, run, install, default default is run
   qw(activity),                                                                 # Optional: Activity name, default is 'Activity'
@@ -53,22 +53,27 @@ if (1)
   qw(keyStorePwd),                                                              # REQUIRED: password of keystore
   qw(log),                                                                      # Output:   message log
   qw(libs),                                                                     # Optional: extra libraries
-  qw(name),                                                                     # Optional: One word name of app, else the name of the folder: '../'
+  qw(name),                                                                     # Optional: One word name of app, default is the name of the folder: '../', will be lower cased and added to the domain name
   qw(parameters),                                                               # Optional: Parameter string to be placed in res for the app
   qw(permissions),                                                              # Optional: Permissions, a standard useful set is applied
   qw(sdk),                                                                      # REQUIRED: Folder containing Android sdk
   qw(sdkLevels),                                                                # Optional: [minSdkVersion,targetSdkVersion], default is [15,25]
   qw(src),                                                                      # Optional: Source of app, default is everything in './src' folder
-  qw(target),                                                                   # Optional: Number of android sdk to use as found via `android list targets`
   qw(title),                                                                    # Optional: Title of app, default is name of app
-  qw(version),                                                                  # Optional: Version ofo app, default is today's date
+  qw(version),                                                                  # Optional: Version of app, default is today's date
  )}
 
+sub appSdkLevels($)                                                             # File name of Android jar for linting
+ {my ($android) = @_;
+  my $l = $android->sdkLevels;
+  return @$l if $l;
+  (15,25)
+ }
+
 sub androidJar($)                                                               # File name of Android jar for linting
  {my ($android) = @_;
   my $sdk = $android->sdk;
-  my $L = $android->sdkLevels;
-  my $l = $L->[-1];
+  my (undef, $l) = $android->appSdkLevels;
   $sdk."platforms/android-$l/android.jar"
  }
 
@@ -84,12 +89,12 @@ sub appTitle
 
 sub sourceFolder                                                                # Folder containing source of app
  {my ($a) = @_;
-  $a->src // $home.'/src';
+  $a->src // $home.'../src';
  }
 
 sub appLibs                                                                     # Folder containing libraries to be copied into the app
  {my ($a) = @_;
-  $a->libs // $home.'/libs';
+  $a->libs // $home.'../libs';
  }
 
 sub package                                                                     # Package for app
@@ -109,7 +114,7 @@ sub apk
 
 sub buildArea($)                                                                # Build folder name
  {my ($a) = @_;
-  $a->buildFolder // $home.'tmp/app/'                                           # Either the user supplied build folder name or the default
+  $a->buildFolder // $home.'../tmp/app/'                                        # Either the user supplied build folder name or the default
  }
 
 sub appBinFolder($)     {my ($a) = @_; $a->buildArea.'bin/'}                    # Bin folder name
@@ -118,8 +123,6 @@ sub appResFolder($)     {my ($a) = @_; $a->buildArea.'res/'}
 sub appSrcFolder($)     {my ($a) = @_; $a->buildArea.'src/'}                    # Source folder name
 sub appLibsFolder($)    {my ($a) = @_; $a->buildArea.'libs/'}                   # Libraries folder
 sub appIcon($)          {my ($a) = @_; $a->buildArea.'icon.png'}                # Icon file name
-sub antPropertiesFile($){my ($a) = @_; $a->buildArea."ant.properties"}          # Ant properties file
-sub antBuildFile($)     {my ($a) = @_; $a->buildArea.'build.xml'}               # Ant build file
 sub manifestFile($)     {my ($a) = @_; $a->buildArea.'AndroidManifest.xml'}     # Name of manifest file
 
 sub logMessage($@)                                                              # Log a message
@@ -185,8 +188,7 @@ sub addPermissions
 sub manifest
  {my ($android) = @_;
   my $permissions = $android->addPermissions;
-
-  my ($minSdk, $targetSdk) = @{$android->sdkLevels};
+  my ($minSdk, $targetSdk) = $android->appSdkLevels;
   my $package     = $android->package;
   my $version     = $android->version;
   my $debug       = $android->debug;
@@ -252,16 +254,17 @@ END
  }
 
 #-------------------------------------------------------------------------------
-# Copy source - could be improved by adding package name automatically
+# Copy source
 #-------------------------------------------------------------------------------
 
 sub copySource
  {my ($android) = @_;
   my $s = $android->sourceFolder;
-  my $p = $android->package  =~ s/\./\//gr;
-  my $t = $android->appSrcFolder."$p/";
+  my $p = $android->package;
+  my $P = $p =~ s/\./\//gr;
+  my $t = $android->appSrcFolder."$P/";
   makePath($t);
-  print STDERR qx(rsync -r $s $t);
+  print STDERR qx(rsync -r $s $t);                                              # Copy source files recursively
   if (my $copy = $android->copyFiles)                                           # Do a copy file step to bring in additional files
    {$copy->($android, $t);
    }
@@ -279,25 +282,6 @@ sub copyLibs
    }
  }
 
-#-------------------------------------------------------------------------------
-# Ant properties plus signing key for Appa Apps
-#-------------------------------------------------------------------------------
-
-sub antProperties
- {my ($android) = @_;
-  my $keyStoreFile = $android->keyStoreFile;
-  my $keyStorePwd  = $android->keyStorePwd;
-  my $ant          = $android->antPropertiesFile;
-  my $s = <<END;
-key.store=$keyStoreFile
-key.alias=vocabulary
-key.store.password=$keyStorePwd
-key.alias.password=$keyStorePwd
-dex.force.jumbo=true
-END
-  writeFile($ant, $s);
- }
-
 #-------------------------------------------------------------------------------
 # Create app
 #-------------------------------------------------------------------------------
@@ -306,19 +290,17 @@ sub create
  {my ($android) = @_;
   my $buildArea = $android->buildArea;
   my $sdk       = $android->sdk;
-  my $target    = $android->target;
   my $name      = $android->appName;
   my $activity  = $android->activity;
   my $package   = $android->package;
   if (-d $buildArea)
-   {my $r = xxx("rm -r $buildArea");                                                    # Remove existing app
+   {my $r = xxx("rm -r $buildArea");                                            # Clear build area
     !$r or confess "Unable to remove existing app build area:\n".
                    "$buildArea\n$r\n";
    }
   $android->pushIcons;                                                          # Create icons
   $android->copySource;                                                         # Copy source
   $android->copyLibs;                                                           # Copy libraries
-  $android->antProperties;                                                      # Ant properties
   $android->manifest;                                                           # Create manifest
   $android->resources;                                                          # Create resources
  }
@@ -330,7 +312,7 @@ sub create
 sub make
  {my ($android) = @_;
 
-  my $app        = $android->appName;
+  my $appName    = $android->appName;
 
   my $sdk        = $android->sdk;
   my $buildTools = $android->buildTools;
@@ -364,9 +346,9 @@ sub make
   my $binRes     = filePath($bin, $res);
   my $classes    = filePath($bin, qw(classes));
 
-  my $api        = $bin."$app.ap_";
-  my $apj        = $bin."$app-unaligned.apk";
-  my $apk        = $bin."$app.apk";
+  my $api        = $bin."$appName.ap_";
+  my $apj        = $bin."$appName-unaligned.apk";
+  my $apk        = $bin."$appName.apk";
 
   if (1)                                                                        # Confirm aapt
    {my $a = xxx("$aapt version");
@@ -611,8 +593,8 @@ use vars qw(@ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);
 
 =head1 Name
 
-Android::Build - Lint, compile, install, start an Android App using just the
-command line tools without using ant or gradle, just 100% Pure Perl
+Android::Build - Lint, compile, install, start an Android App using the command
+line tools minus ant and gradle.
 
 =head1 Prerequisites
 
@@ -633,32 +615,36 @@ contains:
 
  my $a = &Android::Build::new();
 
- $a->title        = qq(Generic App);                                            # Title of the app as seen under the icon
- $a->domain       = qq(com.appaapps);                                           # Domain name in reverse order
- $a->activity     = qq(Activity);                                               # Name of Activity = $activity.java file containing onCreate() for this app
- $a->icon         = qq(~/images/Jets/EEL.jpg);                                  # Image that will be scaled to make an icon using Imagemagick
- $a->sdk          = qq(~/Android/sdk/);                                         # Android SDK on the local machine
- $a->sdkLevels    = [15,25];                                                    # Min sdk, target sdk for manifest
- $a->androidJar   = $a->sdk."platforms/android-25/android.jar";                 # Android sdk jar
- $a->keyAlias     = qq(xxx);                                                    # Alias of key to be used to sign this app
- $a->keyStoreFile = qq(~/keystore/release-key.keystore);                        # Key store file
- $a->keyStorePwd  = qq(xxx);                                                    # Password for keystore
+ $a->sdk          = qq(/home/phil/Android/sdk/);            # Android SDK on the local machine
+ $a->buildTools   = $a->sdk."build-tools/25.0.2/";          # Build tools folder
+ $a->name         = qq(Genapp);                             # Name of the app, this value will be lower cased and appended to the domain name to form the package name
+ $a->title        = qq(Generic App);                        # Title of the app as seen under the icon
+ $a->domain       = qq(com.appaapps);                       # Domain name in reverse order
+ $a->icon         = "$home/images/Jets/EEL.jpg";            # English Electric Lightning: image that will be scaled to make an icon using Imagemagick
+ $a->keyAlias     = qq(xxx);                                # Alias of key to be used to sign this app
+ $a->keyStoreFile = "$home/keystore/release-key.keystore";  # Key store file
+ $a->keyStorePwd  = qq(xxx);                                # Password for key store file
 
- $a->build(qw(run));                                                            # Build, install and run the app on the only emulator
+ $a->build(qw(run));                                        # Build, install and run the app on the only emulator
 
 Modify the values above to reflect your local environment, then start an
 emulator and run:
 
- perl SampleApp/perl/makeWithPerl.pl
+ cd SampleApp/perl/ && perl makeWithPerl.pl
 
 to compile the sample app and load it into the emulator.
 
-If you do not already have a signing key, you can create one with the supplied script:
+If you do not already have a signing key, you can create one with the supplied
+script:
 
- perl SampleApp/perl/generateAKey.pl
+ SampleApp/perl/generateAKey.pl
 
 =head1 File layout
 
+A sample file layout is included in folder:
+
+ SampleApp/
+
 If your Android build description is in file:
 
  /somewhere/$folder/perl/makeWithPerl.pl
@@ -684,15 +670,19 @@ is your reversed domain name written in lowercase. Executing:
  ...
  $a->build(qw(run));
 
-will copy the files in the 𝘀𝗿𝗰 and 𝗹𝗶𝗯 folders into an Android project
-created in the app build folder before starting the build of your app.
+from folder:
+
+ SampleApp/perl/
+
+will copy the files in the 𝘀𝗿𝗰 and 𝗹𝗶𝗯𝘀 folders to the 𝗯𝘂𝗶𝗹𝗱𝗙𝗼𝗹𝗱𝗲𝗿 before
+starting the build of your app.
 
 If this does not meet your requirements, then provide a 𝘀𝘂𝗯
 
  $a->copyFiles = sub ...
 
-which will be called just before the build begins to allow you to copy in any
-other files needed to build your app.
+which will be called just before the build begins to allow you to copy into the
+𝗯𝘂𝗶𝗹𝗱𝗙𝗼𝗹𝗱𝗲𝗿 any other files needed to build your app.
 
 =head1 Parameters
 
@@ -701,7 +691,10 @@ methods:
 
 =head2 activity
 
-Optional: Activity name, default is 'Activity'
+Optional: Activity name, default is 'Activity'. The name of the class to start on the
+emulator is the concatenation of:
+
+ $domain. lc($name). '/.'. $activity
 
 =head2 buildTools
 
@@ -710,7 +703,9 @@ app
 
 =head2 buildFolder
 
-Optional: Name of a folder in which to build the app
+Optional: Name of a folder in which to build the app. The default is ../tmp
+
+This folder will be cleared (without warning) before the app is built.
 
 =head2 copyFiles
 
@@ -718,7 +713,7 @@ Optional: Sub to copy additional files into the app before it is complied
 
 =head2 debug
 
-Optional: Make app debuggable is specified and true
+Optional: Make the app debuggable if specified and true
 
 =head2 device
 
@@ -726,12 +721,15 @@ Optional: Device to run on, default is the only emulator
 
 =head2 domain
 
-REQUIRED: Domain name for app
+REQUIRED: Domain name for app.  The name of the class to start on the emulator
+is the concatenation of:
+
+ $domain. lc($name). '/.'. $activity
 
 =head2 icon
 
-Optional: Jpg file containing a picture that will be scaled to make
-an icon for the app, default is 'icon.jpg'
+Optional: Jpg file containing a picture that will be scaled using Imagemagick
+to make an icon for the app, default is 'icon.jpg'
 
 =head2 keyAlias
 
@@ -739,7 +737,7 @@ REQUIRED: alias used in keytool to name the key to be used to sign this app
 
 =head2 keyStoreFile
 
-REQUIRED: file name of key store
+REQUIRED: name of key store file
 
 =head2 keyStorePwd
 
@@ -747,7 +745,7 @@ REQUIRED: password of key store file
 
 =head2 log
 
-Output:   message log
+Output: message log
 
 =head2 libs
 
@@ -755,19 +753,37 @@ Optional: extra libraries
 
 =head2 name
 
-Optional: One word name of app, else the name of the folder: '../'
+Optional: One word name for the app, default is the name of the folder
+containing the current folder. This name will be lower cased and added to the
+domain name to form the name of the package to be started to run the app. If
+the package name so constructed does not match any package statement in any of
+your java files then your app will not start as expected.
+
+The name of the class to start on the emulator is the concatenation of:
+
+ $domain. lc($name). '/.'. $activity
+
+The apk for the generated app will be:
+
+ $name.'.apk'
 
 =head2 parameters
 
-Optional: Parameter string to be placed in res for the app
+Optional: Parameter string to be placed in folder: 𝗿𝗲𝘀  as a string accessible
+via:
+
+ R.string.parameters
 
 =head2 permissions
 
-Optional: Permissions, a standard useful set is applied
+Optional: Permissions for the app. A standard useful set is supplied by default
+if none are provided.
 
 =head2 sdk
 
-REQUIRED: Folder containing Android sdk
+REQUIRED: Folder containing Android sdk. This information is used in
+conjunction with parameter: 𝘀𝗱𝗸𝗟𝗲𝘃𝗲𝗹𝘀 to find the right Android.jar
+and to access the platform tools folder.
 
 =head2 sdkLevels
 
@@ -775,15 +791,20 @@ Optional: [minSdkVersion,targetSdkVersion], default is [15,25]
 
 =head2 src
 
-Optional: Source of app, default is everything in './src' folder
+Optional: Source of app, default is everything in the '../src' folder.
 
-=head2 target
-
-Optional: Number of android sdk to use as found via `android list targets`
+The source files do not have to be positioned within the domain name hierarchy,
+you can for instance have all your source files at the root of source directory
+or anywhere below it if that is more convenient. This module compiles all java
+files found under the source directory plus any others that have been copied
+into the app 𝗯𝘂𝗶𝗹𝗱𝗙𝗼𝗹𝗱𝗲𝗿 by the 𝗰𝗼𝗽𝘆𝗳𝗶𝗹𝗲 option. The only requirements is that
+each java file use a package statement to declare its final position in the
+domain name hierarchy.
 
 =head2 title
 
-Optional: Title of app, default is the name of app
+Optional: Title of app, default is the name of app, This title will appear
+below the app icon on the Android device display.
 
 =head2 version
 
